@@ -272,14 +272,15 @@ rgb565lsb (GdkImage *image, guchar *pixels, int rowstride, GdkColormap *colormap
 				| (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10;
 #else
 			/* swap endianness first */
-			data = s[1] | s[0] << 8 | s[3] << 16 | s[2] << 24;
-			s += 4;
-			*o++ = (data & 0xf800) | (data & 0xe000) >> 5
-				| (data & 0x7e0) >> 3 | (data & 0x600) >> 9;
-			*o++ = (data & 0x1f) << 11 | (data & 0x1c) << 6
-				| (data & 0xf8000000) >> 24 | (data & 0xe0000000) >> 29;
-			*o++ = (data & 0x7e00000) >> 11 | (data & 0x6000000) >> 17
-				| (data & 0x1f0000) >> 13 | (data & 0x1c0000) >> 18;
+			data = s[1] << 24 | s[0] << 16 | s[3] << 8 | s[2];
+                        s += 4;
+
+			*o++ = (data & 0xf8000000) >> 16 | (data & 0xe0000000) >> 21
+				| (data & 0x7e00000) >> 19 | (data & 0x6000000) >> 25;
+			*o++ = (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10
+				| (data & 0xf800) >> 8 | (data & 0xe000) >> 13;
+			*o++ = (data & 0x7e0) << 5 | (data & 0x600) >> 1
+				| (data & 0x1f) << 3 | (data & 0x1c) >> 2;
 #endif
 		}
 		/* check for last remaining pixel */
@@ -345,12 +346,12 @@ rgb565msb (GdkImage *image, guchar *pixels, int rowstride, GdkColormap *colormap
 				| (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10;
 #else
 			data = *s++;
-			*o++ = (data & 0xf800) | (data & 0xe000) >> 5
-				| (data & 0x7e0) >> 3 | (data & 0x600) >> 9;
-			*o++ = (data & 0x1f) << 11 | (data & 0x1c) << 6
-				| (data & 0xf8000000) >> 24 | (data & 0xe0000000) >> 29;
-			*o++ = (data & 0x7e00000) >> 11 | (data & 0x6000000) >> 17
-				| (data & 0x1f0000) >> 13 | (data & 0x1c0000) >> 18;
+			*o++ = (data & 0xf8000000) >> 16 | (data & 0xe0000000) >> 21
+				| (data & 0x7e00000) >> 19 | (data & 0x6000000) >> 25;
+			*o++ = (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10
+				| (data & 0xf800) >> 8 | (data & 0xe000) >> 13;
+			*o++ = (data & 0x7e0) << 5 | (data & 0x600) >> 1
+				| (data & 0x1f) << 3 | (data & 0x1c) >> 2;
 #endif
 		}
 		/* check for last remaining pixel */
@@ -1032,14 +1033,14 @@ rgbconvert (GdkImage *image, guchar *pixels, int rowstride, int alpha, GdkColorm
 
 /**
  * gdk_pixbuf_get_from_drawable:
- * @dest: Destination pixbuf, or NULL if a new pixbuf should be created.
+ * @dest: Destination pixbuf, or %NULL if a new pixbuf should be created.
  * @src: Source drawable.
  * @cmap: A colormap if @src is a pixmap.  If it is a window, this argument will
  * be ignored.
  * @src_x: Source X coordinate within drawable.
  * @src_y: Source Y coordinate within drawable.
- * @dest_x: Destination X coordinate in pixbuf, or 0 if @dest is NULL.
- * @dest_y: Destination Y coordinate in pixbuf, or 0 if @dest is NULL.
+ * @dest_x: Destination X coordinate in pixbuf, or 0 if @dest is %NULL.
+ * @dest_y: Destination Y coordinate in pixbuf, or 0 if @dest is %NULL.
  * @width: Width in pixels of region to get.
  * @height: Height in pixels of region to get.
  *
@@ -1051,16 +1052,16 @@ rgbconvert (GdkImage *image, guchar *pixels, int rowstride, int alpha, GdkColorm
  * If the drawable is a window, the @cmap argument will be ignored and the
  * window's own colormap will be used instead.
  *
- * If the specified destination pixbuf @dest is #NULL, then this function will
+ * If the specified destination pixbuf @dest is %NULL, then this function will
  * create an RGB pixbuf with 8 bits per channel and no alpha, with the same size
  * specified by the @width and @height arguments.  In this case, the @dest_x and
  * @dest_y arguments must be specified as 0, otherwise the function will return
- * #NULL.  If the specified destination pixbuf is not NULL and it contains alpha
+ * %NULL.  If the specified destination pixbuf is not %NULL and it contains alpha
  * information, then the filled pixels will be set to full opacity.
  *
  * If the specified drawable is a pixmap, then the requested source rectangle
  * must be completely contained within the pixmap, otherwise the function will
- * return #NULL.
+ * return %NULL.
  *
  * If the specified drawable is a window, then it must be viewable, i.e. all of
  * its ancestors up to the root window must be mapped.  Also, the specified
@@ -1070,9 +1071,9 @@ rgbconvert (GdkImage *image, guchar *pixels, int rowstride, int alpha, GdkColorm
  * inferior windows of a different depth than that of the source window will also
  * be undefined.
  *
- * Return value: The same pixbuf as @dest if it was non-NULL, or a newly-created
+ * Return value: The same pixbuf as @dest if it was non-%NULL, or a newly-created
  * pixbuf with a reference count of 1 if no destination pixbuf was specified; in
- * the latter case, NULL will be returned if not enough memory could be
+ * the latter case, %NULL will be returned if not enough memory could be
  * allocated for the pixbuf to be created or if the image data could not be
  * retrieved from the drawable.
  **/
