@@ -25,6 +25,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include "gdk-pixbuf-private.h"
@@ -133,11 +134,10 @@ struct headerpair {
 /* Data needed for the "state" during decompression */
 struct bmp_compression_state {
 	gint phase;
-	gint RunCount;
-
-	guchar *linebuff;
-	gint linebuffsize;	/* these two counts in nibbles */
-	gint linebuffdone;
+	gint run;
+	gint count;
+	gint x, y;
+	guchar *p;
 };
 
 /* Progressive loading */
@@ -225,6 +225,35 @@ gdk_pixbuf__bmp_image_load(FILE *f)
 	return pb;
 }
 
+/* Picks up a 32-bit little-endian integer starting at the specified location.
+ * Does it by hand instead of dereferencing a simple (gint *) cast due to
+ * alignment constraints many platforms.
+ */
+static int
+lsb_32 (guchar *src)
+{
+	return src[0] | (src[1] << 8) | (src[2] << 16) | (src[3] << 24);
+}
+
+/* Same as above, but for 16-bit little-endian integers. */
+static short
+lsb_16 (guchar *src)
+{
+	return src[0] | (src[1] << 8);
+}
+
+static gboolean
+grow_buffer (struct bmp_progressive_state *State)
+{
+  guchar *tmp = realloc (State->buff, State->BufferSize);
+  if (!tmp) {
+    State->read_state = READ_STATE_ERROR;
+    return FALSE;
+  }
+  State->buff = tmp;
+  return TRUE;
+}
+
 static gboolean
 DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 	      struct bmp_progressive_state *State)
@@ -233,19 +262,10 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 
         /* FIXME this is totally unrobust against bogus image data. */
 
-	if (State->BufferSize < GUINT32_FROM_LE (* (guint32 *) &BIH[0]) + 14) {
-		State->BufferSize = GUINT32_FROM_LE (* (guint32 *) &BIH[0]) + 14;
-		State->buff = realloc (State->buff, State->BufferSize);
-		if (State->buff == NULL) {
-#if 0
-			g_set_error (error,
-				     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                     _("Not enough memory to load bitmap image"));
-#endif
-			State->read_state = READ_STATE_ERROR;
+	if (State->BufferSize < lsb_32 (&BIH[0]) + 14) {
+		State->BufferSize = lsb_32 (&BIH[0]) + 14;
+		if (!grow_buffer (State))
 			return FALSE;
-		}
 		return TRUE;
 	}
 
@@ -253,16 +273,16 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 	DumpBIH(BIH);
 #endif
 
-	State->Header.size = GUINT32_FROM_LE (* (guint32 *) &BIH[0]);
+	State->Header.size = lsb_32 (&BIH[0]);
 	if (State->Header.size == 40) {
-		State->Header.width = GINT32_FROM_LE (* (gint32 *) &BIH[4]);
-		State->Header.height = GINT32_FROM_LE (* (gint32 *) &BIH[8]);
-		State->Header.depth = GUINT16_FROM_LE (* (guint16 *) &BIH[14]);
-		State->Compressed = GUINT32_FROM_LE (* (guint32 *) &BIH[16]);
+		State->Header.width = lsb_32 (&BIH[4]);
+		State->Header.height = lsb_32 (&BIH[8]);
+		State->Header.depth = lsb_16 (&BIH[14]);
+		State->Compressed = lsb_32 (&BIH[16]);
 	} else if (State->Header.size == 12) {
-		State->Header.width = GUINT16_FROM_LE (* (guint16 *) &BIH[4]);
-		State->Header.height = GUINT16_FROM_LE (* (guint16 *) &BIH[6]);
-		State->Header.depth = GUINT16_FROM_LE (* (guint16 *) &BIH[10]);
+		State->Header.width = lsb_16 (&BIH[4]);
+		State->Header.height = lsb_16 (&BIH[6]);
+		State->Header.depth = lsb_16 (&BIH[10]);
 		State->Compressed = BI_RGB;
 	} else {
 #if 0
@@ -333,7 +353,9 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 		State->LineWidth = (State->LineWidth / 4) * 4 + 4;
 
 	if (State->pixbuf == NULL) {
-		if (State->Type == 32)
+		if (State->Type == 32 ||
+		    State->Compressed == BI_RLE4 ||
+		    State->Compressed == BI_RLE8)
 			State->pixbuf =
 			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
 					   (gint) State->Header.width,
@@ -361,18 +383,17 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 
 	}
 
-	if (!(State->Compressed == BI_RGB || State->Compressed == BI_BITFIELDS)) {
-		State->compr.linebuffdone = 0;
-		State->compr.linebuffsize = State->Header.width;
-		if (State->Type == 8)
-			State->compr.linebuffsize *= 2;
-		State->compr.linebuff = g_malloc ((State->compr.linebuffsize + 1) / 2);
+	/* make all pixels initially transparent */
+	if (State->Compressed == BI_RLE4 || State->Compressed == BI_RLE8) {
+		memset (State->pixbuf->pixels, 0, State->pixbuf->rowstride * State->Header.height);
+		State->compr.p = State->pixbuf->pixels 
+			+ State->pixbuf->rowstride * (State->Header.height- 1);
 	}
 
 	State->BufferDone = 0;
 	if (State->Type <= 8) {
 		State->read_state = READ_STATE_PALETTE;
-		State->BufferSize = GUINT32_FROM_LE (* (guint32 *) &BFH[10]) - 14 - State->Header.size;
+		State->BufferSize = lsb_32 (&BFH[10]) - 14 - State->Header.size;
 	} else if (State->Compressed == BI_RGB) {
 		State->read_state = READ_STATE_DATA;
 		State->BufferSize = State->LineWidth;
@@ -390,12 +411,13 @@ DecodeHeader (unsigned char *BFH, unsigned char *BIH,
 		return FALSE;
 	}
 
-	State->buff = g_realloc (State->buff, State->BufferSize);
+	if (!grow_buffer (State)) 
+		return FALSE;
 
         return TRUE;
 }
 
-static void
+static gboolean
 DecodeColormap (guchar *buff,
 		struct bmp_progressive_state *State)
 {
@@ -419,8 +441,11 @@ DecodeColormap (guchar *buff,
 		State->BufferSize = 2;
 	else
 		State->BufferSize = State->LineWidth;
+	
+	if (!grow_buffer (State))
+		return FALSE;
 
-	State->buff = g_realloc (State->buff, State->BufferSize);
+	return TRUE;
 }
 
 /* Finds the lowest set bit and the number of set bits */
@@ -439,7 +464,7 @@ find_bits (int n, int *lowest, int *n_set)
 }
 
 /* Decodes the 3 shorts that follow for the bitmasks for BI_BITFIELDS coding */
-static void
+static gboolean
 decode_bitmasks (struct bmp_progressive_state *State, guchar *buf)
 {
 	State->r_mask = buf[0] | (buf[1] << 8) | (buf[2] << 16) | (buf[3] << 24);
@@ -468,7 +493,10 @@ decode_bitmasks (struct bmp_progressive_state *State, guchar *buf)
 	State->read_state = READ_STATE_DATA;
 	State->BufferDone = 0;
 	State->BufferSize = State->LineWidth;
-	State->buff = g_realloc (State->buff, State->BufferSize);
+	if (!grow_buffer (State)) 
+		return FALSE;
+
+	return TRUE;
 }
 
 /*
@@ -494,7 +522,12 @@ gdk_pixbuf__bmp_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 	context->read_state = READ_STATE_HEADERS;
 
 	context->BufferSize = 26;
-	context->buff = g_malloc (26);
+	context->buff = malloc (26);
+	if (!context->buff) {
+		g_free (context);
+		return NULL;
+	}
+
 	context->BufferDone = 0;
 	/* 14 for the BitmapFileHeader, 12 for the BitmapImageHeader */
 
@@ -527,16 +560,13 @@ gdk_pixbuf__bmp_image_stop_load (gpointer data)
 
 	g_return_if_fail(context != NULL);
 
-	if (context->compr.linebuff != NULL)
-		g_free(context->compr.linebuff);
-
 	if (context->Colormap != NULL)
 		g_free(context->Colormap);
 
 	if (context->pixbuf)
 		gdk_pixbuf_unref(context->pixbuf);
 
-	g_free (context->buff);
+	free (context->buff);
 	g_free (context);
 }
 
@@ -682,6 +712,8 @@ OneLine16 (struct bmp_progressive_state *context)
 			*pixels++ = (r << 3) | (r >> 2);
 			*pixels++ = (g << 3) | (g >> 2);
 			*pixels++ = (b << 3) | (b >> 2);
+
+			src += 2;
 		}
 }
 
@@ -808,7 +840,9 @@ OneLine (struct bmp_progressive_state *context)
 	if (context->updated_func != NULL) {
 		(*context->updated_func) (context->pixbuf,
 					  0,
-					  context->Lines,
+					  (context->Header.Negative ?
+					   (context->Lines - 1) :
+					   (context->Header.height - context->Lines)),
 					  context->Header.width,
 					  1,
 					  context->user_data);
@@ -816,133 +850,159 @@ OneLine (struct bmp_progressive_state *context)
 	}
 }
 
-static void
+#define NEUTRAL       0
+#define ENCODED       1
+#define ESCAPE        2   
+#define DELTA_X       3
+#define DELTA_Y       4
+#define ABSOLUTE      5
+#define SKIP          6
+
+#define END_OF_LINE   0
+#define END_OF_BITMAP 1
+#define DELTA         2
+
+static gboolean
 DoCompressed(struct bmp_progressive_state *context)
 {
-	gint count, pos;
-	switch (context->compr.phase) {
-	case 0:		/* Neutral state */
-		if (context->buff[0] != 0) {	/* run count */
-			context->compr.RunCount = context->buff[0];
-			if (context->Type == 8)
-				context->compr.RunCount *= 2;
-			while (context->compr.RunCount > 0) {
-				if (context->compr.linebuffdone & 1) {
-					guchar *ptr = context->compr.linebuff +
-					    context->compr.linebuffdone / 2;
+	gint i, j;
+	gint y;
+	guchar c;
+	gint idx;
 
-					*ptr = (*ptr & 0xF0) | (context->buff[1] >> 4);
-					context->buff[1] = (context->buff[1] << 4) |
-							   (context->buff[1] >> 4);
-					context->compr.linebuffdone++;
-					context->compr.RunCount--;
-				}
+	if (context->compr.y >= context->Header.height)
+		return TRUE;
 
-				if (context->compr.RunCount) {
-					count = context->compr.linebuffsize -
-					    context->compr.linebuffdone;
-					if (count > context->compr.RunCount)
-						count = context->compr.RunCount;
+	y = context->compr.y;
 
-					memset (context->compr.linebuff +
-						context->compr.linebuffdone / 2,
-						context->buff[1],
-						(count + 1) / 2);
-					context->compr.RunCount -= count;
-					context->compr.linebuffdone += count;
-				}
-				if (context->compr.linebuffdone == context->compr.linebuffsize) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
+ 	for (i = 0; i < context->BufferSize; i++) {
+		c = context->buff[i];
+		switch (context->compr.phase) {
+		    case NEUTRAL:
+			    if (c) {
+				    context->compr.run = c;
+				    context->compr.phase = ENCODED;
+			    }
+			    else
+				    context->compr.phase = ESCAPE;
+			    break;
+		    case ENCODED:
+			    for (j = 0; j < context->compr.run; j++) {
+				    if (context->Compressed == BI_RLE8)
+					    idx = c;
+				    else if (j & 1) 
+					    idx = c & 0x0f;
+				    else 
+					    idx = (c >> 4) & 0x0f;
+				    if (context->compr.x < context->Header.width) {
+					    *context->compr.p++ = context->Colormap[idx][2];
+					    *context->compr.p++ = context->Colormap[idx][1];
+					    *context->compr.p++ = context->Colormap[idx][0];
+					    *context->compr.p++ = 0xff;
+					    context->compr.x++;    
+				    }
+			    }
+			    context->compr.phase = NEUTRAL;
+			    break;
+		    case ESCAPE:
+			    switch (c) {
+				case END_OF_LINE:
+					context->compr.x = 0;
+					context->compr.y++;
+					context->compr.p = context->pixbuf->pixels 
+						+ (context->pixbuf->rowstride * (context->Header.height - context->compr.y - 1))
+						+ (4 * context->compr.x);
+					context->compr.phase = NEUTRAL;
+					break;
+				case END_OF_BITMAP:
+					context->compr.x = 0;
+					context->compr.y = context->Header.height;
+					context->compr.phase = NEUTRAL;
+					break;
+				case DELTA:
+					context->compr.phase = DELTA_X;
+					break;
+				default:
+					context->compr.run = c;
+					context->compr.count = 0;
+					context->compr.phase = ABSOLUTE;
+					break;
+			    }
+			    break;
+		    case DELTA_X:
+			    context->compr.x += c;
+			    context->compr.phase = DELTA_Y;
+			    break;
+		    case DELTA_Y:
+			    context->compr.y += c;
+			    context->compr.p = context->pixbuf->pixels 
+				    + (context->pixbuf->rowstride * (context->Header.height - context->compr.y - 1))
+				    + (4 * context->compr.x);
+			    context->compr.phase = NEUTRAL;
+			    break;
+		    case ABSOLUTE:
+			    if (context->Compressed == BI_RLE8) {
+				    idx = c;
+				    if (context->compr.x < context->Header.width) {
+					    *context->compr.p++ = context->Colormap[idx][2];
+					    *context->compr.p++ = context->Colormap[idx][1];
+					    *context->compr.p++ = context->Colormap[idx][0];
+					    *context->compr.p++ = 0xff;
+					    context->compr.x++;    
+				    }
+				    context->compr.count++;
 
-					if (context->compr.linebuffdone & 1)
-						context->buff[1] = (context->buff[1] << 4) |
-								   (context->buff[1] >> 4);
-					context->compr.linebuffdone = 0;
-				}
-			}
-		} else {	/* Escape */
-			if (context->buff[1] == 0) {	/* End of line */
-				if (context->compr.linebuffdone) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
+				    if (context->compr.count == context->compr.run) {
+					    if (context->compr.run & 1)
+						    context->compr.phase = SKIP;
+					    else
+						    context->compr.phase = NEUTRAL;
+				    }
+			    }
+			    else {
+				    for (j = 0; j < 2; j++) {
+					    if (context->compr.count & 1)
+						    idx = c & 0x0f;
+					    else 
+						    idx = (c >> 4) & 0x0f;
+					    if (context->compr.x < context->Header.width) {
+						    *context->compr.p++ = context->Colormap[idx][2];
+						    *context->compr.p++ = context->Colormap[idx][1];
+						    *context->compr.p++ = context->Colormap[idx][0];
+						    *context->compr.p++ = 0xff;
+						    context->compr.x++;    
+					    }
+					    context->compr.count++;
 
-					context->compr.linebuffdone = 0;
-				}
-			} else if (context->buff[1] == 1) {	/* End of image */
-				if (context->compr.linebuffdone) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
-				}
-
-				context->compr.phase = 2;
-			} else if (context->buff[1] == 2)	/* Cursor displacement */
-				;	/* not implemented */
-			else {
-				context->compr.phase = 1;
-				context->compr.RunCount = context->buff[1];
-				if (context->Type == 8)
-					context->compr.RunCount *= 2;
-				context->BufferSize = (context->compr.RunCount + 3) / 4 * 2;
-				context->buff = g_realloc (context->buff, context->BufferSize);
-			}
+					    if (context->compr.count == context->compr.run) {
+						    if ((context->compr.run & 3) == 1
+							|| (context->compr.run & 3) == 2) 
+							    context->compr.phase = SKIP;
+						    else
+							    context->compr.phase = NEUTRAL;
+						    break;
+					    }
+				    }
+			    }
+			    break;
+		    case SKIP:
+			    context->compr.phase = NEUTRAL;
+			    break;
 		}
-		context->BufferDone = 0;
-		break;
-	case 1:
-		pos = 0;
-		while (pos < context->compr.RunCount) {
-			count = context->compr.linebuffsize - context->compr.linebuffdone;
-			if (count > context->compr.RunCount)
-				count = context->compr.RunCount;
-
-			if ((context->compr.linebuffdone & 1) || (pos & 1)) {
-				gint i, newval;
-				guchar *ptr;
-				for (i = 0; i < count; i++) {
-					ptr = context->compr.linebuff + (i +
-					      context->compr.linebuffdone) / 2;
-					newval = *(context->buff + (pos + i) / 2) & (0xf0 >> (((pos + i) % 2) * 4));
-					if (((pos + i) % 2) ^ ((context->compr.linebuffdone + i) % 2)) {
-						if ((pos + i) % 2)
-							newval <<= 4;
-						else
-							newval >>= 4;
-					}
-					*ptr = (*ptr & (0xf << (((i + context->compr.linebuffdone) % 2) * 4))) | newval;
-				}
-			} else {
-				memmove (context->compr.linebuff +
-					 context->compr.linebuffdone / 2,
-					 context->buff + pos / 2,
-					 (count + 1) / 2);
-			}
-			pos += count;
-			context->compr.linebuffdone += count;
-			if (context->compr.linebuffdone == context->compr.linebuffsize) {
-				guchar *tmp = context->buff;
-				context->buff = context->compr.linebuff;
-				OneLine (context);
-				context->buff = tmp;
-
-				context->compr.linebuffdone = 0;
-			}
-		}
-		context->compr.phase = 0;
-		context->BufferSize = 2;
-		context->buff = g_realloc (context->buff, context->BufferSize);
-		context->BufferDone = 0;
-		break;
-	case 2:
-		context->BufferDone = 0;
-		break;
 	}
+	if (context->updated_func != NULL) {
+		if (context->compr.y > y)
+			(*context->updated_func) (context->pixbuf,
+						  0,
+						  y,
+						  context->Header.width,
+						  context->compr.y - y,
+						  context->user_data);
+
+	}
+
+	context->BufferDone = 0;
+	return TRUE;
 }
 
 /*
@@ -991,18 +1051,20 @@ gdk_pixbuf__bmp_image_load_increment (gpointer data, guchar *buf, guint size)
 			break;
 
 		case READ_STATE_PALETTE:
-			DecodeColormap (context->buff, context);
+			if (!DecodeColormap (context->buff, context))
+				return FALSE;
 			break;
 
 		case READ_STATE_BITMASKS:
-			decode_bitmasks (context, context->buff);
+			if (!decode_bitmasks (context, context->buff))
+				return FALSE;
 			break;
 
 		case READ_STATE_DATA:
 			if (context->Compressed == BI_RGB || context->Compressed == BI_BITFIELDS)
 				OneLine (context);
-			else
-				DoCompressed (context);
+			else if (!DoCompressed (context))
+				return FALSE;
 
 			break;
 
