@@ -255,7 +255,7 @@ gdk_pixbuf__jpeg_image_load (FILE *f)
 	if (sigsetjmp (jerr.setjmp_buffer, 1)) {
 		/* Whoops there was a jpeg error */
 		if (pixbuf)
-			g_object_unref (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
 
 		jpeg_destroy_decompress (&cinfo);
 		return NULL;
